#!/usr/bin/perl -w
use strict;
use Fcntl ':flock';
#####################
# BS Counter (v2.0.2)
# Copyright 1999-2003 Brian Stanback <brian at stanback dot net>
# This software is protected under the GNU General Public License (see LICENSE)

#
# Set config options (see the docs/README for config instructions)
#

# WWW path where png digit images are located (include trailing slash)
my $digits_dir = "/cgi-bin/bscounter/images/digits/";

# Directory that houses the xml data files (include trailing slash)
my $data_dir = "/home/sites/www.stanback.net/web/cgi-bin/bscounter/data/";

# Put your ip here if you don't want the counter to count hits from you (otherwise leave it as 127.0.0.1)
my $my_ip = "127.0.0.1";

#
# Begin script
#

my @time = localtime(time);
my $total;
my @today;
my %geographic;
my %browsers;
my %platforms;
my %terms;
my %referrers;
my @hosts;

my %in;
&getvars;
my $file = $in{'page'} || "default";
my $style = $in{'style'} || "default";
my $type = $in{'type'} || "text";
my $imgtype = $in{'imgtype'} || "png";
my $nbdigits = $in{'nbdigits'} || 1;
my $allow_reloads = $in{'reloads'} || 0;

# List of Browsers (Name => Regex)
my @agents = (
		"MSN => MSN",
		"Opera => Opera",
		"AOL => AOL",
		"Internet Explorer 5+ => MSIE [5-9]",
		"Internet Explorer 4- => MSIE",
		"Netscape 6+ => Netscape",
		"Netscape 4- => Nav",
		"Apple/Safari => Safari",
		"Nokia => Nokia",
		"Robot/Spider => Googlebot|Surf|Crawler|Mercator|Seek|Arachnoidea|Link|Slurp|Copier|Shrelock",
		"Konqueror => Konqueror",
		"WebTV => WebTV",
		"Mosaic => NCSA",
		"Lynx => Lynx",
		"Download Agents => Kontiki|Wget|GetRight|gozilla",
		"Mozilla => Mozilla",
		"Unknown => .*"
	);

# List of Platforms
my @os = (
		"Windows NT/2K/XP => Windows NT|Windows 2000|Windows XP|WinNT",
		"Windows 9x/ME => Windows|Win98|Win95",
		"Windows 3.1 => Win3.1",
		"Mac OSX => Mac OS X",
		"Mac PowerPC => Mac",
		"Linux => Linux",
		"HP-UX => HP-UX",
		"SunOS => SunOS",
		"BSD => BSD",
		"UNIX => UNIX",
		"IRIX => IRIX",
		"QNX => QNX",
		"OS/2 => OS/2",
		"Nokia => Nokia",
		"Amiga => Amiga",
		"WebTV => WebTV",
		"Unknown => .*"
	);

#
# Parse the XML stats file
#

if (-f "$data_dir$file.xml") {
	&parse_xml("$data_dir$file.xml");
}

#
# Update stats
#

if ($ENV{'REMOTE_ADDR'} and $ENV{'REMOTE_ADDR'} ne $my_ip) {
	if (!$ENV{'REMOTE_HOST'}) {
		my @subnet_numbers = split (/\./, $ENV{'REMOTE_ADDR'});
		$ENV{'REMOTE_HOST'} = gethostbyaddr(pack("C4", @subnet_numbers), 2) || $ENV{'REMOTE_ADDR'};
	}
	if ($allow_reloads != 1) {
		$allow_reloads = 1;
		foreach my $host (@hosts) {
			if ($host eq $ENV{'REMOTE_HOST'}) { $allow_reloads = 0; }
		}
	}
	if ($allow_reloads) {
		$total++;

		if ($today[0] eq "$time[3]$time[5]") { $today[1]++; }
		else { $today[1] = 1; }

		if ($ENV{'REMOTE_HOST'} eq $ENV{'REMOTE_ADDR'}) {
			$geographic{'ip'}++;
		} else {
			my @hostname = split(/\./, $ENV{'REMOTE_HOST'});
			$geographic{lc($hostname[$#hostname])}++;
		}

		if ($ENV{'HTTP_USER_AGENT'}) {
			foreach my $agent (@agents) {
				my ($name, $regex) = split(' => ', $agent);
				if ($ENV{'HTTP_USER_AGENT'} =~ /$regex/i) { $browsers{$name}++; last; }
			}
			foreach my $platform (@os) {
				my ($name, $regex) = split(' => ', $platform);
				if ($ENV{'HTTP_USER_AGENT'} =~ /$regex/i) { $platforms{$name}++; last; }
			}
		}

		if ($ENV{'HTTP_REFERER'} and $ENV{'HTTP_REFERER'} =~ m!^(https?://)([-a-z0-9\.]{4,})((?::\d+)?)(/[^#?]+(?:#\S+)?)\?([^#?]+(?:#\S+)?)$!i) {
			$referrers{lc("$1$2$3$4")}++;
			foreach my $term (get_terms(lc($5))) {
				$term =~ s/^[\-\+\?]|[\-\+\?]$//;
				if ($term =~ /^[A-Za-z0-9\+\-\?]{2,20}$/) {
					$terms{$term}++;
				}
			}
		}

		unshift(@hosts, $ENV{'REMOTE_HOST'});

		open(XML, ">$data_dir$file.xml");
		flock(XML, LOCK_EX);
		seek(XML, 0, 0);
		truncate(XML, 0);
		print XML "<?xml version=\"1.0\"?>\n";
		print XML "<counter>\n";
		print XML "\t<total>$total</total>\n";
		print XML "\t<today time=\"$time[3]$time[5]\">$today[1]</today>\n";
		print XML "\t<geographic>\n";
		foreach my $tld (keys %geographic) {
			print XML "\t\t<country tld=\"$tld\" hits=\"$geographic{$tld}\" />\n";
		}
		print XML "\t</geographic>\n";
		print XML "\t<browsers>\n";
		foreach my $agent (keys %browsers) {
			print XML "\t\t<agent name=\"$agent\" hits=\"$browsers{$agent}\" />\n";
		}
		print XML "\t</browsers>\n";
		print XML "\t<platforms>\n";
		foreach my $os (keys %platforms) {
			print XML "\t\t<os name=\"$os\" hits=\"$platforms{$os}\" />\n";
		}
		print XML "\t</platforms>\n";
		print XML "\t<terms>\n";
		foreach my $term (keys %terms) {
			print XML "\t\t<term value=\"$term\" hits=\"$terms{$term}\" />\n";
		}
		print XML "\t</terms>\n";
		print XML "\t<referrers>\n";
		foreach my $ref (keys %referrers) {
			print XML "\t\t<ref url=\"$ref\" hits=\"$referrers{$ref}\" />\n";
		}
		print XML "\t</referrers>\n";
		print XML "\t<hosts>\n";
		foreach my $host (@hosts[0..19]) {
			if ($host) {
				print XML "\t\t<host value=\"$host\" />\n";
			}
		}
		print XML "\t</hosts>\n";
		print XML "</counter>";
		close(XML);
	}
}

#
# Print out the count
#
print "Content-type: text/html\n\n";
if ($type eq "image") {
	$total = '0' x ($nbdigits - length($total)) . $total;
	my @mydigits = split('', $total);
	foreach my $num (@mydigits) {
		print "<img src=\"$digits_dir$style/$num.$imgtype\" alt=\"$num\" />";
	}
} else {
	while ($total =~ s/(.*\d)(\d\d\d)/$1,$2/) {}
	print $total;
}

#
# XML Parsing Routines
#

sub parse_xml {
	open(XML, $_[0]);
	flock(XML, LOCK_EX);
	while (my $line = <XML>) {
		if ($line =~ /<ref url=\"(.*)\" hits=\"(\d+)\" \/>/) { $referrers{$1} = $2; }
		elsif ($line =~ /<host value=\"(.*)\" \/>/) { push(@hosts, $1); }
		elsif ($line =~ /<term value=\"(.*)\" hits=\"(\d+)\" \/>/) { $terms{$1} = $2; }
		elsif ($line =~ /<country tld=\"(.*)\" hits=\"(\d+)\" \/>/) { $geographic{$1} = $2; }
		elsif ($line =~ /<agent name=\"(.*)\" hits=\"(\d+)\" \/>/) { $browsers{$1} = $2; }
		elsif ($line =~ /<os name=\"(.*)\" hits=\"(\d+)\" \/>/) { $platforms{$1} = $2; }
		elsif ($line =~ /<today time=\"(\d+)\">(\d+)<\/today>/) { $today[0] = $1; $today[1] = $2; }
		elsif ($line =~ /<total>(\d+)<\/total>/) { $total = $1; }
	}
	close(XML);
}

#
# Get query string configuration variables (strip everything that isn't a number or letter)
#

sub getvars {
	if ($ENV{QUERY_STRING}) {
		for (split(/\&/, $ENV{'QUERY_STRING'})) {
			my($key, $val) = split /=/;
			$val =~ s/%([0-9a-fA-F]{2})/chr(hex($1))/ge;
			$val =~ s/[^[A-Za-z0-9_-]//g;
			$in{$key} = $val;
		}
	}
}

sub get_terms {
	for (split /\&/, $_[0]) {
		my($key, $val) = split /=/;
		if ($key and $key =~ /^(q|p|query)$/) {
			$val =~ tr/+/ /;
			$val =~ s/%([0-9a-fA-F]{2})/chr(hex($1))/ge;
			return split(' ', $val);
		}
	}
}
