#!/usr/bin/perl -w
use strict;
use Fcntl ':flock';
#####################
# BS Counter (v2.0.2)
# Copyright 1999-2003 Brian Stanback <brian at stanback dot net>
# This software is protected under the GNU General Public License (see LICENSE)

#
# Set config options (see the docs/README for config instructions)
#

# Directory that houses the xml data files (include trailing slash)
my $data_dir = "./data/";

# Stats page header html
my $header = "./templates/header.html";

# Stats page footer html
my $footer = "./templates/footer.html";

# WWW path to the image directory (include trailing slash)
my $image_dir = "images/";

#
# Begin script
#

my @time = localtime(time);
my $total;
my @today;
my %geographic; my $geographic_total = 0;
my %browsers; my $browsers_total = 0;
my %platforms; my $platforms_total = 0;
my %terms; my $terms_total = 0;
my %referrers; my $referrers_total = 0;
my @hosts;

my %in;
&getvars;
my $file = $in{'page'};

# Descriptions for all the crazy top-level domains out there
my %domains = (
		com => "Int/Commercial",
		org => "Int/Organization",
		net => "Int/Network",
		gov => "US Government",
		mil => "US Military",
		edu => "US Educational",
		int => "International",
		aero => "Air-transport industry",
		biz => "Int/Business",
		coop => "Int/Cooperatives",
		info => "Int/Informative",
		museum => "Int/Museum",
		pro => "Int. Professional",
		af => "Afghanistan",
		al => "Albania",
		dz => "Algeria",
		as => "American Somoa",
		ad => "Andorra",
		ao => "Angola",
		ai => "Anguilla",
		aq => "Antarctica",
		ag => "Antigua and Barbuda",
		ar => "Argentina",
		am => "Armenia",
		aw => "Aruba",
		ac => "Ascension Island", 
		au => "Australia",
		at => "Austria",
		az => "Azerbaijan",
		bh => "Bahrain",
		bd => "Bangladesh",
		bb => "Barbados",
		by => "Belarus",
		be => "Belgium",
		bz => "Belize",
		bj => "Benin",
		bm => "Bermuda",
		bt => "Bhutan",
		bo => "Bolivia",
		ba => "Bosnia and Herzegovina",
		bw => "Botswana",
		bv => "Bouvet Island",
		br => "Brazil",
		io => "British Indian Ocean Territory",
		bn => "Brunei",
		bg => "Bulgaria",
		bf => "Burkina Faso",
		bi => "Burundi",
		kh => "Cambodia",
		cm => "Cameroon",
		ca => "Canada",
		cv => "Cape Verde",
		cf => "Central African Republic",
		td => "Chad",
		gg => "Channel Islands - Guernsey",
		je => "Channel Islands - Jersey",
		cl => "Chile",
		cn => "China",
		cx => "Christmas Island",
		cc => "Cocos (Keeling) Islands",
		co => "Colombia",
		km => "Comoros",
		cg => "Congo",
		ck => "Cook Islands",
		cr => "Costa Rica",
		ci => "Coted Ivoire",
		hr => "Croatia",
		cu => "Cuba",
		cy => "Cyprus",
		cz => "Czech Republic",
		dk => "Denmark",
		dj => "Djibouti",
		dm => "Dominica",
		do => "Dominican Republic",
		tp => "East Timor",
		ec => "Ecuador",
		eg => "Egypt",
		sv => "El Salvador",
		gq => "Equatorial Guinea",
		er => "Eritrea",
		ee => "Estonia",
		et => "Ethiopia",
		fk => "Falklands Islands",
		fo => "Faroe Islands",
		fj => "Fiji",
		fi => "Finland",
		fr => "France",
		gf => "French Guiana",
		pf => "French Polynesia",
		tf => "French Southern Territories",
		fx => "French Metropolitan",
		ga => "Gabon",
		ge => "Georgia",
		de => "Germany",
		gh => "Ghana",
		gi => "Gibraltar",
		gr => "Greece",
		gl => "Greenland",
		gd => "Grenada",
		gp => "Guadeloupe",
		gu => "Guam",
		gt => "Guatemala",
		gn => "Guinea",
		gy => "Guyana",
		ht => "Haiti",
		hm => "Heardand McDonald Islands",
		hn => "Honduras",
		hk => "HongKong",
		hu => "Hungary",
		is => "Iceland",
		in => "India",
		id => "Indonesia",
		ir => "Iran",
		iq => "Iraq",
		ie => "Ireland",
		il => "Israel",
		it => "Italy",
		jm => "Jamaica",
		jp => "Japan",
		jo => "Jordan",
		kz => "Kazakhstan",
		ke => "Kenya",
		ki => "Kiribati",
		kp => "Peoples Republic of Democratic Korea",
		kr => "Republic of Korea",
		kw => "Kuwait",
		kg => "Kyrgyzstan",
		la => "Laos",
		lv => "Latvia",
		lb => "Lebanon",
		ls => "Lesotho",
		lr => "Liberia",
		ly => "Libyan Arab Jamahiriya",
		li => "Liechtenstein",
		lt => "Lithuania",
		lu => "Luxembourg",
		mo => "Macao",
		mk => "Macedonia",
		mw => "Malawi",
		my => "Malaysia",
		ml => "Mali",
		mt => "Malta",
		mh => "Marshall Islands",
		mq => "Martinique",
		mr => "Mauritania",
		mu => "Mauritius",
		yt => "Mayotte",
		mx => "Mexico",
		fm => "Micronesia",
		md => "Moldova",
		mc => "Monaco",
		mn => "Mongolia",
		ms => "Montserrat",
		ma => "Morocco",
		mz => "Mozambique",
		mm => "Myanmar",
		na => "Namibia",
		nr => "Nauru",
		np => "Nepal",
		nl => "Netherlands",
		an => "Netherlands Antilles",
		nc => "New Caledonia",
		nz => "New Zealand",
		ni => "Nicaragua",
		ne => "Niger",
		ng => "Nigeria",
		nu => "Niue",
		nf => "Norfolk Island",
		mp => "Northern Mariana Islands",
		no => "Norway",
		om => "Oman",
		pk => "Pakistan",
		pw => "Palau",
		pa => "Panama",
		pq => "Papua New Guinea",
		py => "Paraguay",
		pe => "Peru",
		ph => "Philippines",
		pn => "Pitcairn",
		pl => "Poland",
		pt => "Portugal",
		pr => "Puerto Rico",
		qa => "Qatar",
		re => "Reunion",
		ro => "Romania",
		ru => "Russia",
		rw => "Rwanda",
		kn => "Saint Kitts and Nevis",
		lc => "Saint Lucia",
		vc => "Saint Vincent and the Grenadines",
		ws => "Samoa",
		sm => "San Marino",
		st => "Sao Tomeand Prin cipe",
		sa => "Saudi Arabia",
		sn => "Senegal",
		sc => "Seychelles",
		sl => "Sierra Leone",
		sg => "Singapore",
		sk => "Slovakia",
		si => "Slovenia",
		sb => "Solomon Islands",
		so => "Somalia",
		za => "South Africa",
		gs => "South Georgia and the South Sandwich Islands",
		es => "Spain",
		lk => "Sri Lanka",
		sh => "St. Helena",
		pm => "St. Pierreand Miquelon",
		sd => "Sudan",
		sr => "Suriname",
		sj => "Svalbardand Jan Mayen Islands",
		sz => "Swaziland",
		se => "Sweden",
		ch => "Switzerland",
		sy => "Syria",
		tw => "Taiwan",
		tj => "Tajikistan",
		tz => "Tanzania",
		th => "Thailand",
		bs => "The Bahamas",
		ky => "The Cayman Islands",
		to => "Tonga",
		tt => "Trinidad and Tobago",
		tn => "Tunisia",
		tk => "Tokelau",
		tr => "Turkey",
		tm => "Turkmenistan",
		tc => "Turks and Caicos Islands",
		tv => "Tuvalu",
		ug => "Uganda",
		ua => "Ukraine",
		ae => "United Arab Emirates",
		uk => "United Kingdom",
		us => "United States",
		um => "United States Minor Outlying Islands",
		uy => "Urugauy",
		uz => "Uzbekistan",
		vu => "Vanuatu",
		va => "Vatican City State",
		ve => "Venezuela",
		vn => "Vietnam",
		vg => "Virgin Islands (British)",
		vi => "Virgin Islands (US)",
		wf => "Wallis and Futuna Islands",
		eh => "Western Sahara",
		ye => "Yemen",
		yu => "Yugoslavia",
		zr => "Zaire",
		zm => "Zambia",
		zw => "Zimbabwe",
		ip => "Int/IP Address"
	);

# Print the header
print "Content-type: text/html\n\n";
open(FILE, $header);
if ($file) {
	while(<FILE>) { $_ =~ s/<!--page-->/Counter Stats for <span class=\"g\">$file<\/span>/gi; print; }
} else {
	while(<FILE>) { $_ =~ s/<!--page-->/Counter Stats/gi; print; }
}
close(FILE);

if ($file and -f "$data_dir$file.xml") {
	#
	# Parse the XML stats file and print the stats
	#

	&parse_xml("$data_dir$file.xml");

	#
	# Grab the totals so we can generate percentages
	#

	foreach my $agent (keys %browsers) {
		$browsers_total += $browsers{$agent};
	}
	foreach my $os (keys %platforms) {
		$platforms_total += $platforms{$os};
	}
	foreach my $tld (keys %geographic) {
		$geographic_total += $geographic{$tld};
	}
	foreach my $term (keys %terms) {
		$terms_total += $terms{$term};
	}

	#
	# Print the html
	#

	while ($total =~ s/(.*\d)(\d\d\d)/$1,$2/) {}
	if ($today[0] eq "$time[3]$time[5]") {
		while ($today[1] =~ s/(.*\d)(\d\d\d)/$1,$2/) {}
	} else {
		$today[1] = 0;
	}

	print "<table width=\"578\" cellspacing=\"0\" cellpadding=\"0\">\n";
	print "<tr><td>Total Hits</td><td>$total</td></tr>\n";
	print "<tr><td>Hits Today</td><td>$today[1]</td></tr>\n";
	print "<tr><th colspan=\"2\">Browser Stats</th></tr>\n";
	foreach my $agent (sort {$browsers{$b} <=> $browsers{$a}} keys %browsers) {
		print "<tr><td>$agent</td><td>".mkbar($browsers{$agent}, $browsers_total, "purple", 1)."</td></tr>\n";
	}
	print "<tr><th colspan=\"2\">Platform Stats</th></tr>\n";
	foreach my $os (sort {$platforms{$b} <=> $platforms{$a}} keys %platforms) {
		print "<tr><td>$os</td><td>".mkbar($platforms{$os}, $platforms_total, "red", 1)."</td></tr>\n";
	}
	print "<tr><th colspan=\"2\">Top 20 Countries</th></tr>\n";
	my $i = 0;
	foreach my $tld (sort {$geographic{$b} <=> $geographic{$a}} keys %geographic) {
		if ($i == 20) { last; }
		if (!$domains{$tld}) { $domains{$tld} = $tld; }
		print "<tr><td>$domains{$tld}</td><td>".mkbar($geographic{$tld}, $geographic_total, "green", 1)."</td></tr>\n";
		$i++;
	}
	if (scalar keys(%terms) > 0) {
		my $i = 0;
		my ($term2, $term3);
		print "<tr><th colspan=\"2\">Top 20 Search Words</th></tr>\n";
		foreach my $term (sort {$terms{$b} <=> $terms{$a}} keys %terms) {
			if ($i == 20) { last; }
			print "<tr><td>$term</td><td>".mkbar($terms{$term}, $terms_total, "orange", 2)."</td></tr>\n";
			$i++;
		}
	}
	if (scalar keys(%referrers) > 0) {
		my $i = 0;
		my ($term2, $term3);
		print "<tr><th colspan=\"2\">Top 20 Referrers</th></tr>\n";
		foreach my $ref (sort {$referrers{$b} <=> $referrers{$a}} keys %referrers) {
			if ($i == 20) { last; }
			print "<tr><td colspan=\"2\"><a href=\"$ref\">$ref ($referrers{$ref})</a></td></tr>\n";
			$i++;
		}
	}
	print "<tr><th colspan=\"2\">Last 20 Accesses</th></tr>\n";
	foreach my $host (@hosts) {
		print "<tr><td colspan=\"2\">$host</td></tr>\n";
	}
	print "</table>\n";
} else {
	if (-d $data_dir and !$file) {
		print "View stats for...<br /><br />\n";
		opendir(DATA, $data_dir);
		while (my $file = readdir(DATA)) {
			if ($file =~ s/\.xml$//g) {
				print "&#183; <a href=\"$ENV{'SCRIPT_URI'}?page=$file\">$file</a><br />\n";
			}
		}
		closedir(DATA);
	} else {
		print "Error: Cannot open the specified counter. This may be because you have not yet placed the counter on your webpage, once the counter receives the first hit, the stats will become available.";
	}
}

# Print the footer
open(FILE, $footer);
while(<FILE>) { print; }
close(FILE);

#
# XML Parsing Routines
#

sub parse_xml {
	open(XML, $_[0]);
	flock(XML, LOCK_EX);
	while (my $line = <XML>) {
		if ($line =~ /<ref url=\"(.*)\" hits=\"(\d+)\" \/>/) { $referrers{$1} = $2; }
		elsif ($line =~ /<host value=\"(.*)\" \/>/) { push(@hosts, $1); }
		elsif ($line =~ /<term value=\"(.*)\" hits=\"(\d+)\" \/>/) { $terms{$1} = $2; }
		elsif ($line =~ /<country tld=\"(.*)\" hits=\"(\d+)\" \/>/) { $geographic{$1} = $2; }
		elsif ($line =~ /<agent name=\"(.*)\" hits=\"(\d+)\" \/>/) { $browsers{$1} = $2; }
		elsif ($line =~ /<os name=\"(.*)\" hits=\"(\d+)\" \/>/) { $platforms{$1} = $2; }
		elsif ($line =~ /<today time=\"(\d+)\">(\d+)<\/today>/) { $today[0] = $1; $today[1] = $2; }
		elsif ($line =~ /<total>(\d+)<\/total>/) { $total = $1; }
	}
	close(XML);
}

#
# Print the progress bar
#

sub mkbar {
	my ($hits, $total, $color, $type) = @_;
	my $percent = int(($hits / $total) * 1000) / 10;
	my $width = int(3.8 * $percent) + 1;
	if ($type == 1) {
		if ($percent == 0) { $percent = "&lt;0.1"; }
		return "<img src=\"$image_dir$color-lcap.gif\" height=\"12\" width=\"6\" /><img src=\"$image_dir$color-bar.gif\" height=\"12\" width=\"$width\" /><img src=\"$image_dir$color-rcap.gif\" height=\"12\" width=\"6\" /> $percent%";
	} else {
		return "<img src=\"$image_dir$color-lcap.gif\" height=\"12\" width=\"6\" /><img src=\"$image_dir$color-bar.gif\" height=\"12\" width=\"$width\" /><img src=\"$image_dir$color-rcap.gif\" height=\"12\" width=\"6\" /> $hits";
	}
}

#
# Get query string configuration variables (strip everything that isn't a number or letter)
#

sub getvars {
	if ($ENV{QUERY_STRING}) {
		for (split /\&/, $ENV{QUERY_STRING}) {
			my($key, $val) = split /=/;
			$val =~ s/%([0-9a-fA-F]{2})/chr(hex($1))/ge;
			$val =~ s/[^\w_-]//g;
			$in{$key} = $val;
		}
	}
}
